/*
Module : EZViewDoc.cpp
Purpose: Implements the MFC document class for EZView

Copyright (c) 2000 - 2022 by PJ Naughter (Web: www.naughter.com, Email: pjna@naughter.com)

All rights reserved.

Copyright / Usage Details:

You are allowed to include the source code in any product (commercial, shareware, freeware or otherwise)
when your product is released in binary form. You are allowed to modify the source code in any way you want
except you cannot modify the copyright details at the top of each module. If you want to distribute source
code with your application, then you are only allowed to distribute versions released by the author. This is
to maintain a single distribution point for the source code.

*/


//////////////////// Includes /////////////////////////////////////////////////

#include "stdafx.h"
#include "EZView.h"
#include "RenameDlg.h"
#include "EZViewDoc.h"
#include "EZViewView.h"
#include "GotoPageDlg.h"


//////////////////// Implementation ///////////////////////////////////////////

#pragma warning(suppress: 26409 26433 26440 26477)
IMPLEMENT_DYNCREATE(CEZViewDoc, CDocument)

#pragma warning(suppress: 26433 26440)
BEGIN_MESSAGE_MAP(CEZViewDoc, CDocument) //NOLINT(clang-diagnostic-unused-local-typedef, modernize-avoid-c-arrays)
  ON_UPDATE_COMMAND_UI(ID_FILE_PRINT, &CEZViewDoc::OnUpdateFilePrint)
  ON_UPDATE_COMMAND_UI(ID_FILE_PRINT_PREVIEW, &CEZViewDoc::OnUpdateFilePrintPreview)
  ON_UPDATE_COMMAND_UI(ID_FILE_SEND_MAIL, &CEZViewDoc::OnUpdateFileSendMail)
  ON_COMMAND(ID_FILE_RENAME, &CEZViewDoc::OnFileRename)
  ON_UPDATE_COMMAND_UI(ID_FILE_RENAME, &CEZViewDoc::OnUpdateFileRename)
  ON_COMMAND(ID_FILE_COPY, &CEZViewDoc::OnFileCopy)
  ON_UPDATE_COMMAND_UI(ID_FILE_COPY, &CEZViewDoc::OnUpdateFileCopy)
  ON_COMMAND(ID_FILE_DELETE, &CEZViewDoc::OnFileDelete)
  ON_UPDATE_COMMAND_UI(ID_FILE_DELETE, &CEZViewDoc::OnUpdateFileDelete)
  ON_COMMAND(ID_FILE_MOVE, &CEZViewDoc::OnFileMove)
  ON_UPDATE_COMMAND_UI(ID_FILE_MOVE, &CEZViewDoc::OnUpdateFileMove)
  ON_COMMAND(ID_FILE_PROPERTIES, &CEZViewDoc::OnFileProperties)
  ON_UPDATE_COMMAND_UI(ID_FILE_PROPERTIES, &CEZViewDoc::OnUpdateFileProperties)
  ON_COMMAND(ID_TOOLS_SETASWALLPAPER, &CEZViewDoc::OnToolsSetAsWallpaper)
  ON_UPDATE_COMMAND_UI(ID_TOOLS_SETASWALLPAPER, &CEZViewDoc::OnUpdateToolsSetAsWallpaper)
  ON_COMMAND(ID_FILE_SEND_MAIL, &CEZViewDoc::OnFileSendMail)
  ON_UPDATE_COMMAND_UI(ID_FILE_SEND_MAIL, &CEZViewDoc::OnUpdateFileSendMail)
  ON_COMMAND(ID_VIEW_FIRSTIMAGE, &CEZViewDoc::OnViewFirstImage)
  ON_UPDATE_COMMAND_UI(ID_VIEW_FIRSTIMAGE, &CEZViewDoc::OnUpdateViewFirstImage)
  ON_COMMAND(ID_VIEW_LASTIMAGE, &CEZViewDoc::OnViewLastImage)
  ON_UPDATE_COMMAND_UI(ID_VIEW_LASTIMAGE, &CEZViewDoc::OnUpdateViewLastImage)
  ON_COMMAND(ID_VIEW_NEXTIMAGE, &CEZViewDoc::OnViewNextImage)
  ON_UPDATE_COMMAND_UI(ID_VIEW_NEXTIMAGE, &CEZViewDoc::OnUpdateViewNextImage)
  ON_COMMAND(ID_VIEW_PREVIOUSIMAGE, &CEZViewDoc::OnViewPreviousImage)
  ON_UPDATE_COMMAND_UI(ID_VIEW_PREVIOUSIMAGE, &CEZViewDoc::OnUpdateViewPreviousImage)
  ON_COMMAND(ID_VIEW_REFRESH, &CEZViewDoc::OnViewRefresh)
  ON_UPDATE_COMMAND_UI(ID_VIEW_REFRESH, &CEZViewDoc::OnUpdateViewRefresh)
  ON_COMMAND(ID_VIEW_RANDOMIMAGE, &CEZViewDoc::OnViewRandomImage)
  ON_UPDATE_COMMAND_UI(ID_VIEW_RANDOMIMAGE, &CEZViewDoc::OnUpdateViewRandomImage)
  ON_COMMAND(ID_EDIT_COPY, &CEZViewDoc::OnEditCopy)
  ON_UPDATE_COMMAND_UI(ID_EDIT_COPY, &CEZViewDoc::OnUpdateEditCopy)
  ON_COMMAND(ID_VIEW_FIRSTPAGE, &CEZViewDoc::OnViewFirstPage)
  ON_UPDATE_COMMAND_UI(ID_VIEW_FIRSTPAGE, &CEZViewDoc::OnUpdateViewFirstPage)
  ON_COMMAND(ID_VIEW_LASTPAGE, &CEZViewDoc::OnViewLastPage)
  ON_UPDATE_COMMAND_UI(ID_VIEW_LASTPAGE, &CEZViewDoc::OnUpdateViewLastPage)
  ON_COMMAND(ID_VIEW_NEXTPAGE, &CEZViewDoc::OnViewNextPage)
  ON_UPDATE_COMMAND_UI(ID_VIEW_NEXTPAGE, &CEZViewDoc::OnUpdateViewNextPage)
  ON_COMMAND(ID_VIEW_PREVIOUSPAGE, &CEZViewDoc::OnViewPreviousPage)
  ON_UPDATE_COMMAND_UI(ID_VIEW_PREVIOUSPAGE, &CEZViewDoc::OnUpdateViewPreviousPage)
  ON_COMMAND(ID_VIEW_OPENPAGE, &CEZViewDoc::OnViewOpenPage)
  ON_UPDATE_COMMAND_UI(ID_VIEW_OPENPAGE, &CEZViewDoc::OnUpdateViewOpenPage)
  ON_COMMAND(ID_VIEW_STARTANIMATION, &CEZViewDoc::OnViewStartAnimation)
  ON_UPDATE_COMMAND_UI(ID_VIEW_STARTANIMATION, &CEZViewDoc::OnUpdateViewStartAnimation)
END_MESSAGE_MAP()

CEZViewDoc::CEZViewDoc() noexcept //NOLINT(modernize-use-equals-default)
{
}

#pragma warning(suppress: 26440)
void CEZViewDoc::Serialize(CArchive& /*ar*/)
{
  ASSERT(false); //Serialization is handled at a higher level
}

BOOL CEZViewDoc::OnOpenDocument(LPCTSTR lpszPathName)
{
  DeleteContents();

  //Validate our parameters
#pragma warning(suppress: 26429)
  CEZViewView* pView{GetView()};
  ASSERT(pView != nullptr);
#pragma warning(suppress: 26489)
  ASSERT(!pView->m_pImage); //NOLINT(clang-analyzer-core.CallAndMessage)

  //Load up the new file
  pView->StopAnimation(); //NOLINT(clang-analyzer-core.CallAndMessage)
  bool bLoadError{false};

  if (!pView->CreateRenderTargetIfNecessary())
    return FALSE;
#pragma warning(suppress: 26429)
  CRenderTarget* pRenderTarget{pView->GetRenderTarget()};
  ASSERT(pRenderTarget != nullptr);
#pragma warning(suppress: 26489)
  pView->m_pImage.reset();
  pView->GetImageDetails(lpszPathName);
  CString sExt;
  _tsplitpath_s(lpszPathName, nullptr, 0, nullptr, 0, nullptr, 0, sExt.GetBuffer(_MAX_EXT), _MAX_EXT);
  sExt.ReleaseBuffer();
  if (sExt.CompareNoCase(_T(".QOI")) == 0)
  {
    if (FAILED(pView->LoadQOI(lpszPathName, pRenderTarget->GetRenderTarget(), pView->m_pImage)))
    {
      pView->m_pImage.reset();
      bLoadError = true;
    }
    else
      pView->m_ImageType = CEZViewView::IMAGE_TYPE::SINGLE_IMAGE;
  }
  else
  {
    if (FAILED(pView->LoadNormalFrame(0, pRenderTarget->GetRenderTarget(), pView->m_pImage)))
    {
      pView->m_pImage.reset();
      bLoadError = true;
    }
  }

  if (bLoadError)
  {
    pView->KillSlideshow();
    CString sMsg;
    AfxFormatString1(sMsg, IDS_FAILED_TO_OPEN_DOC, lpszPathName);
    AfxMessageBox(sMsg);
    return FALSE;
  }

  return TRUE;
}

void CEZViewDoc::DeleteContents()
{
  CEZViewView* pView{GetView()};
  if (pView != nullptr)
#pragma warning(suppress: 26489)
    pView->m_pImage.reset();

  //Let the parent class do its thing
  __super::DeleteContents();
}

#pragma warning(suppress: 26429)
void CEZViewDoc::OnUpdateFilePrint(CCmdUI* pCmdUI)
{
#pragma warning(suppress: 26486 26489)
  pCmdUI->Enable(GetView()->m_pImage != nullptr);
}

#pragma warning(suppress: 26429)
void CEZViewDoc::OnUpdateFilePrintPreview(CCmdUI* pCmdUI)
{
#pragma warning(suppress: 26486 26489)
  pCmdUI->Enable(GetView()->m_pImage != nullptr);
}

#pragma warning(suppress: 26429 26434)
void CEZViewDoc::OnUpdateFileSendMail(CCmdUI* pCmdUI)
{
#pragma warning(suppress: 26486 26489)
  pCmdUI->Enable(GetView()->m_pImage != nullptr);
}

void CEZViewDoc::OnFileRename()
{
  //Get the name of the current file
  CString sOriginalPath{GetPathName()};
  CString sDrive;
  CString sDir;
  CString sFname;
  CString sExt;
  _tsplitpath_s(sOriginalPath, sDrive.GetBuffer(_MAX_DRIVE), _MAX_DRIVE, sDir.GetBuffer(_MAX_DIR), _MAX_DIR, sFname.GetBuffer(_MAX_FNAME), _MAX_FNAME, sExt.GetBuffer(_MAX_EXT), _MAX_EXT);
  sExt.ReleaseBuffer();
  sFname.ReleaseBuffer();
  sDir.ReleaseBuffer();
  sDrive.ReleaseBuffer();
  CString sPath;
  _tmakepath_s(sPath.GetBuffer(_MAX_PATH), _MAX_PATH, nullptr, nullptr, sFname, sExt);
  sPath.ReleaseBuffer();

  //Bring up the rename dialog
  CRenameDlg dlg;
  dlg.m_sFilename = sPath;
  if (dlg.DoModal() == IDOK)
  {
    //Let the shell do the rename
    SHFILEOPSTRUCT shfo{};
    shfo.hwnd = AfxGetMainWnd()->GetSafeHwnd();
    shfo.wFunc = FO_RENAME;
    shfo.fFlags = FOF_ALLOWUNDO;
    CString sFrom{sOriginalPath};
    const int nFromLength{sFrom.GetLength()};
#pragma warning(suppress: 26472)
    std::vector<TCHAR> pszFrom{static_cast<size_t>(nFromLength) + 2, std::allocator<TCHAR>{}};
    _tcscpy_s(pszFrom.data(), pszFrom.size(), sFrom);
#pragma warning(suppress: 26446)
    pszFrom[static_cast<size_t>(nFromLength) + 1] = _T('\0');
    shfo.pFrom = pszFrom.data();
    _tsplitpath_s(dlg.m_sFilename, nullptr, 0, nullptr, 0, sFname.GetBuffer(_MAX_FNAME), _MAX_FNAME, sExt.GetBuffer(_MAX_EXT), _MAX_EXT);
    sExt.ReleaseBuffer();
    sFname.ReleaseBuffer();
    _tmakepath_s(sPath.GetBuffer(_MAX_PATH), _MAX_PATH, sDrive, sDir, sFname, sExt);
    sPath.ReleaseBuffer();
    CString sTo{sPath};
    const int nToLength{sTo.GetLength()};
#pragma warning(suppress: 26472)
    std::vector<TCHAR> pszTo{static_cast<size_t>(nToLength) + 2, std::allocator<TCHAR>{}};
    _tcscpy_s(pszTo.data(), pszTo.size(), sTo);
#pragma warning(suppress: 26446)
    pszTo[static_cast<size_t>(nToLength) + 1] = _T('\0');
    shfo.pTo = pszTo.data();
    const int nSuccess{SHFileOperation(&shfo)};
    if (nSuccess == 0)
    {
      //Set the document title to the new filename
      SetPathName(sPath);

      //Also update the image list
      theApp.RefreshImageList2(sPath);
    }
  }
}

#pragma warning(suppress: 26429)
void CEZViewDoc::OnUpdateFileRename(CCmdUI* pCmdUI)
{
#pragma warning(suppress: 26486 26489)
  pCmdUI->Enable(GetView()->m_pImage != nullptr);
}

void CEZViewDoc::OnFileCopy()
{
  //Get the name of the current file
  CString sPath{GetPathName()};

  //Prompt the user for where they want to copy the file to
  CString sTitle;
  if (!sTitle.LoadString(IDS_FOLDER_COPY_TITLE))
    return;
  CString sDir;
  if (theApp.GetShellManager()->BrowseForFolder(sDir, AfxGetMainWnd(), nullptr, sTitle, BIF_RETURNONLYFSDIRS, nullptr))
  {
    //Ensure the directory has a "\" at its end
    const int nLen{sDir.GetLength()};
    if (sDir[nLen - 1] != _T('\\'))
      sDir += _T("\\");

    //Let the shell do the copy
    SHFILEOPSTRUCT shfo{};
    shfo.hwnd = AfxGetMainWnd()->GetSafeHwnd();
    shfo.wFunc = FO_COPY;
    shfo.fFlags = FOF_ALLOWUNDO;
    CString sFrom{sPath};
    const int nFromLength{sFrom.GetLength()};
#pragma warning(suppress: 26472)
    std::vector<TCHAR> pszFrom{static_cast<size_t>(nFromLength) + 2, std::allocator<TCHAR>{}};
    _tcscpy_s(pszFrom.data(), pszFrom.size(), sFrom);
#pragma warning(suppress: 26446)
    pszFrom[static_cast<size_t>(nFromLength) + 1] = _T('\0');
    shfo.pFrom = pszFrom.data();
    CString sDrive;
    _tsplitpath_s(sDir, sDrive.GetBuffer(_MAX_DRIVE), _MAX_DRIVE, sDir.GetBuffer(_MAX_DIR), _MAX_DIR, nullptr, 0, nullptr, 0);
    sDir.ReleaseBuffer();
    sDrive.ReleaseBuffer();
    CString sFname;
    CString sExt;
    _tsplitpath_s(sPath, nullptr, 0, nullptr, 0, sFname.GetBuffer(_MAX_FNAME), _MAX_FNAME, sExt.GetBuffer(_MAX_EXT), _MAX_EXT);
    sExt.ReleaseBuffer();
    sFname.ReleaseBuffer();
    _tmakepath_s(sPath.GetBuffer(_MAX_PATH), _MAX_PATH, sDrive, sDir, sFname, sExt);
    sPath.ReleaseBuffer();
    CString sTo{sPath};
    const int nToLength{sTo.GetLength()};
#pragma warning(suppress: 26472)
    std::vector<TCHAR> pszTo{static_cast<size_t>(nToLength) + 2, std::allocator<TCHAR>{}};
    _tcscpy_s(pszTo.data(), pszTo.size(), sTo);
#pragma warning(suppress: 26446)
    pszTo[static_cast<size_t>(nToLength) + 1] = _T('\0');
    shfo.pTo = pszTo.data();
    const int nSuccess{SHFileOperation(&shfo)};
    if (nSuccess == 0)
    {
      //Set the document title to the new filename
      SetPathName(sPath);

      //Also update the image list
      theApp.RefreshImageList2(sPath);
    }
  }
}

#pragma warning(suppress: 26429)
void CEZViewDoc::OnUpdateFileCopy(CCmdUI* pCmdUI)
{
#pragma warning(suppress: 26486 26489)
  pCmdUI->Enable(GetView()->m_pImage != nullptr);
}

void CEZViewDoc::OnFileDelete()
{
  //Create a Multi SZ string with the file to delete
  CString sPath{GetPathName()};
#pragma warning(suppress: 26472)
  const size_t nChars{static_cast<size_t>(sPath.GetLength()) + 2};

  //Let the shell do the delete
  SHFILEOPSTRUCT shfo{};
  const CWnd* pMainWnd{AfxGetMainWnd()};
#pragma warning(suppress: 26496)
  AFXASSUME(pMainWnd != nullptr);
  shfo.hwnd = pMainWnd->GetSafeHwnd(); //NOLINT(clang-analyzer-core.CallAndMessage)
  shfo.wFunc = FO_DELETE;

  //Undo is not allowed if the SHIFT key is held down
  if (!(GetKeyState(VK_SHIFT) & 0x8000))
    shfo.fFlags = FOF_ALLOWUNDO;

  //Should we display any UI
  if (!theApp.m_bAskMeOnDelete)
    shfo.fFlags |= FOF_NOCONFIRMATION;

  std::vector<TCHAR> pszFrom{nChars, std::allocator<TCHAR>{}};
  _tcscpy_s(pszFrom.data(), pszFrom.size(), sPath);
#pragma warning(suppress: 26446)
  pszFrom[nChars - 1] = _T('\0');
  shfo.pFrom = pszFrom.data();

  //Let the shell perform the actual deletion
  const int nSuccess{SHFileOperation(&shfo)};
  if (nSuccess == 0)
  {
    //Since we have deleted the current file
    //move onto the next file in the directory
    pMainWnd->SendMessage(WM_COMMAND, ID_VIEW_NEXTIMAGE, 0L);
  }
}

#pragma warning(suppress: 26429)
void CEZViewDoc::OnUpdateFileDelete(CCmdUI* pCmdUI)
{
#pragma warning(suppress: 26486 26489)
  pCmdUI->Enable(GetView()->m_pImage != nullptr);
}

void CEZViewDoc::OnFileMove()
{
  //Get the name of the current file
  CString sPath{GetPathName()};

  //Prompt the user for where they want to copy the file to
  CString sTitle;
  if (!sTitle.LoadString(IDS_FOLDER_COPY_TITLE))
    return;
  CString sDir;
  if (theApp.GetShellManager()->BrowseForFolder(sDir, AfxGetMainWnd(), nullptr, sTitle, BIF_RETURNONLYFSDIRS, nullptr))
  {
    //Ensure the directory has a "\" at its end
    const int nLen{sDir.GetLength()};
    if (sDir[nLen - 1] != _T('\\'))
      sDir += _T("\\");

    //Let the shell do the copy
    SHFILEOPSTRUCT shfo{};
    shfo.hwnd = AfxGetMainWnd()->GetSafeHwnd();
    shfo.wFunc = FO_MOVE;
    shfo.fFlags = FOF_ALLOWUNDO;
    CString sFrom{sPath};
    const int nFromLength{sFrom.GetLength()};
#pragma warning(suppress: 26472)
    std::vector<TCHAR> pszFrom{static_cast<size_t>(nFromLength) + 2, std::allocator<TCHAR>{}};
    _tcscpy_s(pszFrom.data(), pszFrom.size(), sFrom);
#pragma warning(suppress: 26446)
    pszFrom[static_cast<size_t>(nFromLength) + 1] = _T('\0');
    shfo.pFrom = pszFrom.data();
    CString sDrive;
    CString sFname;
    CString sExt;
    _tsplitpath_s(sDir, sDrive.GetBuffer(_MAX_DRIVE), _MAX_DRIVE, sDir.GetBuffer(_MAX_DIR), _MAX_DIR, nullptr, 0, nullptr, 0);
    sDir.ReleaseBuffer();
    sDrive.ReleaseBuffer();
    _tsplitpath_s(sPath, nullptr, 0, nullptr, 0, sFname.GetBuffer(_MAX_FNAME), _MAX_FNAME, sExt.GetBuffer(_MAX_EXT), _MAX_EXT);
    sExt.ReleaseBuffer();
    sFname.ReleaseBuffer();
    _tmakepath_s(sPath.GetBuffer(_MAX_PATH), _MAX_PATH, sDrive, sDir, sFname, sExt);
    CString sTo{sPath};
    const int nToLength{sTo.GetLength()};
#pragma warning(suppress: 26472)
    std::vector<TCHAR> pszTo{static_cast<size_t>(nToLength) + 2, std::allocator<TCHAR>{}};
    _tcscpy_s(pszTo.data(), pszTo.size(), sTo);
#pragma warning(suppress: 26446)
    pszTo[static_cast<size_t>(nToLength) + 1] = _T('\0');
    shfo.pTo = pszTo.data();
    const int nSuccess{SHFileOperation(&shfo)};
    if (nSuccess == 0)
    {
      //Set the document title to the new filename
      SetPathName(sPath);

      //Also update the image list
      theApp.RefreshImageList2(sPath);
    }
  }
}

#pragma warning(suppress: 26429)
void CEZViewDoc::OnUpdateFileMove(CCmdUI* pCmdUI)
{
#pragma warning(suppress: 26486 26489)
  pCmdUI->Enable(GetView()->m_pImage != nullptr);
}

void CEZViewDoc::OnFileProperties()
{
  CString sPathName{GetPathName()};
  SHELLEXECUTEINFO sei;
  memset(&sei, 0, sizeof(sei));
  sei.cbSize = sizeof(sei);
  sei.hwnd = AfxGetMainWnd()->GetSafeHwnd();
  sei.nShow = SW_SHOW;
  sei.lpFile = sPathName.GetBuffer(sPathName.GetLength());
  sei.lpVerb = _T("properties");
  sei.fMask = SEE_MASK_INVOKEIDLIST;
#pragma warning(suppress: 26486)
  ShellExecuteEx(&sei);
  sPathName.ReleaseBuffer();
}

#pragma warning(suppress: 26429)
void CEZViewDoc::OnUpdateFileProperties(CCmdUI* pCmdUI)
{
#pragma warning(suppress: 26486 26489)
  pCmdUI->Enable(GetView()->m_pImage != nullptr);
}

void CEZViewDoc::OnToolsSetAsWallpaper()
{
#pragma warning(suppress: 26474 26492)
  SystemParametersInfo(SPI_SETDESKWALLPAPER, 0, const_cast<PVOID>(static_cast<const void*>(GetPathName().GetString())), SPIF_UPDATEINIFILE | SPIF_SENDCHANGE);
}

#pragma warning(suppress: 26429)
void CEZViewDoc::OnUpdateToolsSetAsWallpaper(CCmdUI* pCmdUI)
{
#pragma warning(suppress: 26486 26489)
  pCmdUI->Enable(GetView()->m_pImage != nullptr);
}

HGLOBAL CEZViewDoc::DDBToDIB(_In_ const CBitmap* pBitmap)
{
  //Validate our parameters
#pragma warning(suppress: 26496)
  AFXASSUME(pBitmap != nullptr);

  //Get info about the DDB
  BITMAP bm{};
  pBitmap->GetObject(sizeof(bm), &bm);

  //Initialize the BITMAPINFOHEADER
  BITMAPINFOHEADER bi{};
  bi.biSize = sizeof(BITMAPINFOHEADER);
  bi.biWidth = bm.bmWidth;
  bi.biHeight = bm.bmHeight;
  bi.biPlanes = 1;
#pragma warning(suppress: 26472)
  bi.biBitCount = static_cast<WORD>(bm.bmPlanes * bm.bmBitsPixel);
  bi.biCompression = BI_RGB;

  //Compute the size of the infoheader and the color table
  int nColors{0};
  if (bi.biBitCount <= 8)
    nColors = (1 << bi.biBitCount);
  SIZE_T dwLen{bi.biSize + (nColors * sizeof(RGBQUAD))};

  //We need a device context to get the DDB from
  HDC hDC{GetDC(nullptr)};
  if (hDC == nullptr)
  {
    TRACE(_T("CEZViewDoc::DDBToDIB, Failed in call to GetDC, Error:%u\n"), GetLastError());
    return nullptr;
  }

  //Allocate enough memory to hold BITMAPINFOHEADER and color table
  auto lpBi{static_cast<LPBITMAPINFOHEADER>(GlobalAlloc(GMEM_FIXED, dwLen))};
  if (lpBi == nullptr)
  {
    TRACE(_T("CEZViewDoc::DDBToDIB, Failed to allocate memory for the DIB, Error:%u\n"), GetLastError());
    ReleaseDC(nullptr, hDC);
    return nullptr;
  }

  //Transfer the BITMAPINFOHEADER into the freshly allocated memory handle.
  memcpy_s(lpBi, dwLen, &bi, dwLen);

  //Call GetDibBits with a nullptr lpBits, so the device driver will
  //calculate the biSizeImage field
  auto hBitmap{static_cast<HBITMAP>(pBitmap->GetSafeHandle())};
  if (hBitmap == nullptr)
  {
    TRACE(_T("CEZViewDoc::DDBToDIB, Failed to get bitmap handle. Error:%u\n"), GetLastError());
    GlobalFree(lpBi);
    ReleaseDC(nullptr, hDC);
    return nullptr;
  }
#pragma warning(suppress: 26490)
  GetDIBits(hDC, hBitmap, 0L, bi.biHeight, nullptr, reinterpret_cast<LPBITMAPINFO>(lpBi), DIB_RGB_COLORS);

  //Transfer back to the stack variable
  memcpy_s(&bi, sizeof(bi), lpBi, sizeof(BITMAPINFOHEADER));

  //If the driver did not fill in the biSizeImage field, then compute it.
  //Each scan line of the image is aligned on a DWORD (32 bit) boundary
  if (bi.biSizeImage == 0)
    bi.biSizeImage = ((((bi.biWidth * bi.biBitCount) + 31) & ~31) / 8) * bi.biHeight;

  //Realloc the buffer so that it can hold all the bits
  dwLen += bi.biSizeImage;
  HGLOBAL hNew{GlobalReAlloc(lpBi, dwLen, GMEM_MOVEABLE)};
  if (hNew == nullptr)
  {
    TRACE(_T("CEZViewDoc::DDBToDIB, Failed to realloc the memory for the DIB, Error:%u\n"), GetLastError());
#pragma warning(suppress: 6001)
    GlobalFree(lpBi);
    ReleaseDC(nullptr, hDC);
    return nullptr;
  }

  //Lock down the memory
  lpBi = static_cast<LPBITMAPINFOHEADER>(GlobalLock(hNew));
  if (lpBi == nullptr)
  {
    TRACE(_T("CEZViewDoc::DDBToDIB, Failed to lock the memory for the DIB, Error:%u\n"), GetLastError());
    GlobalFree(hNew);
    ReleaseDC(nullptr, hDC);
    return nullptr;
  }

  //Finally get the DIB
#pragma warning(suppress: 26481 26490)
  LPBYTE lpBits{reinterpret_cast<LPBYTE>(lpBi) + (bi.biSize + (nColors * sizeof(RGBQUAD)))};
#pragma warning(suppress: 26490)
  if (!GetDIBits(hDC, hBitmap, 0, bi.biHeight, lpBits, reinterpret_cast<LPBITMAPINFO>(lpBi), DIB_RGB_COLORS))
  {
    TRACE(_T("FCEZViewDoc::DDBToDIB, Failed to get the DDB's bits, Error:%u\n"), GetLastError());
    GlobalFree(hNew);
    ReleaseDC(nullptr, hDC);
    return nullptr;
  }

  ReleaseDC(nullptr, hDC);

  return hNew;
}

void CEZViewDoc::OnViewFirstImage()
{
  //Display the hourglass as this may take some time
  CWaitCursor wait;

  //Refresh the image list for this folder if it is currently empty
  if (theApp.m_ImagesInCurrentFolder.size() == 0)
    theApp.RefreshImageList2(GetPathName());

  //Now display the first image
  if (theApp.m_ImagesInCurrentFolder.size())
  {
    CString sOldPathName{GetPathName()};
    const CDocument* pDoc{theApp.OpenDocumentFile(0)};
    if (pDoc == nullptr)
    {
      //Reset this document to the current image before the failed load above
      CDocTemplate* pTemplate{GetDocTemplate()};
#pragma warning(suppress: 26496)
      AFXASSUME(pTemplate != nullptr);
      if (pTemplate->OpenDocumentFile(sOldPathName, FALSE, TRUE) == nullptr)
        pTemplate->OpenDocumentFile(nullptr, FALSE, TRUE);
    }
  }
  else
  {
    GetView()->KillSlideshow();
    AfxMessageBox(IDS_FOLDER_DOES_NOT_CONTAIN_ANY_IMAGES);
  }
}

void CEZViewDoc::OnViewRefresh()
{
  //Display the hourglass as this may take some time
  CWaitCursor wait;

  //Refresh the image list for this folder
  CString sPath{GetPathName()};
  theApp.RefreshImageList2(sPath);

  //Find the current image index after doing the refresh
  const auto iterFind{std::find(theApp.m_ImagesInCurrentFolder.begin(), theApp.m_ImagesInCurrentFolder.end(), sPath)};
  int nNewIndex{-1};
  if (iterFind != theApp.m_ImagesInCurrentFolder.end())
#pragma warning(suppress: 26472 26486)
    nNewIndex = static_cast<int>(iterFind - theApp.m_ImagesInCurrentFolder.begin());

  //Redisplay this image
  CString sOldPathName{GetPathName()};
  CDocTemplate* pTemplate{GetDocTemplate()};
#pragma warning(suppress: 26496)
  AFXASSUME(pTemplate != nullptr);
  pTemplate->OpenDocumentFile(nullptr, FALSE, TRUE);
  CDocument* pDoc{nullptr};
  if (nNewIndex != -1)
    pDoc = theApp.OpenDocumentFile(nNewIndex);
  if (pDoc == nullptr)
  {
    //Reset this document to the current image before the failed load above
    if (pTemplate->OpenDocumentFile(sOldPathName, FALSE, TRUE) == nullptr)
      pTemplate->OpenDocumentFile(nullptr, FALSE, TRUE);
  }
}

#pragma warning(suppress: 26429)
void CEZViewDoc::OnUpdateViewRefresh(CCmdUI* pCmdUI)
{
#pragma warning(suppress: 26486 26489)
  pCmdUI->Enable(GetView()->m_pImage != nullptr);
}

void CEZViewDoc::OnViewLastImage()
{
  //Display the hourglass as this may take some time
  CWaitCursor wait;

  //Refresh the image list for this folder if it is currently empty
  CString sCurrentPath{GetPathName()};
  if (theApp.m_ImagesInCurrentFolder.size() == 0)
    theApp.RefreshImageList2(sCurrentPath);

  //Now display the last image
#pragma warning(suppress: 26472)
  const int nSize{static_cast<int>(theApp.m_ImagesInCurrentFolder.size())};
  if (nSize)
  {
    CString sOldPathName{GetPathName()};
    const CDocument* pDoc{theApp.OpenDocumentFile(nSize - 1)};
    if (pDoc == nullptr)
    {
      //Reset this document to the current image before the failed load above
      CDocTemplate* pTemplate{GetDocTemplate()};
#pragma warning(suppress: 26496)
      AFXASSUME(pTemplate != nullptr);
      if (pTemplate->OpenDocumentFile(sOldPathName, FALSE, TRUE) == nullptr)
        pTemplate->OpenDocumentFile(nullptr, FALSE, TRUE);
    }
  }
  else
  {
    GetView()->KillSlideshow();
    AfxMessageBox(IDS_FOLDER_DOES_NOT_CONTAIN_ANY_IMAGES);
  }
}

void CEZViewDoc::OnViewNextImage()
{
  //Display the hourglass as this may take some time
  CWaitCursor wait;

  //Refresh the image list for this folder if it is currently empty
  CString sCurrentPath{GetPathName()};
  if (theApp.m_ImagesInCurrentFolder.size() == 0)
    theApp.RefreshImageList2(sCurrentPath);

  //Find the current image index if unknown
  if (theApp.m_nCurrentImageIndex == -1)
  {
    const auto iterFind{std::find(theApp.m_ImagesInCurrentFolder.begin(), theApp.m_ImagesInCurrentFolder.end(), sCurrentPath)};
    if (iterFind != theApp.m_ImagesInCurrentFolder.end())
#pragma warning(suppress: 26472 26486)
      theApp.m_nCurrentImageIndex = static_cast<int>(iterFind - theApp.m_ImagesInCurrentFolder.begin());
  }

  //If we found this image then display the next one
#pragma warning(suppress: 26472)
  if (theApp.m_nCurrentImageIndex < static_cast<int>(theApp.m_ImagesInCurrentFolder.size() - 1))
  {
    CString sOldPathName{GetPathName()};
    const CDocument* pDoc{theApp.OpenNextDocumentFile()};
    if (pDoc == nullptr)
    {
      //Reset this document to the current image before the failed load above
      CDocTemplate* pTemplate{GetDocTemplate()};
#pragma warning(suppress: 26496)
      AFXASSUME(pTemplate != nullptr);
      if (pTemplate->OpenDocumentFile(sOldPathName, FALSE, TRUE) == nullptr)
        pTemplate->OpenDocumentFile(nullptr, FALSE, TRUE);
    }
  }
  else
  {
    //We've reached the last image, ask if they want to go back to the begining
    bool bShowFirst{false};
    if (theApp.m_bPromptStartEndFolder)
    {
      GetView()->KillSlideshow();

      //Ask the user if we can go the last image
      CString sContent;
      if (!sContent.LoadString(IDS_REACHED_LAST_IMAGE_IN_FOLDER))
        return;
      CTaskDialog dlg{sContent, _T(""), theApp.m_pszAppName, TDCBF_YES_BUTTON | TDCBF_NO_BUTTON};
      dlg.SetOptions(TDF_ALLOW_DIALOG_CANCELLATION | TDF_POSITION_RELATIVE_TO_WINDOW);
      CString sDontAskAgain;
      if (!sDontAskAgain.LoadString(IDS_DONT_ASK_AGAIN))
        return;
      dlg.SetVerificationCheckboxText(sDontAskAgain);
      bShowFirst = (dlg.DoModal(AfxGetMainWnd()->GetSafeHwnd()) == IDYES);
      theApp.m_bPromptStartEndFolder = !dlg.GetVerificationCheckboxState();
    }
    else
      bShowFirst = true;

    if (bShowFirst)
      OnViewFirstImage();
  }
}

void CEZViewDoc::OnViewPreviousImage()
{
  //Display the hourglass as this may take some time
  CWaitCursor wait;

  //Refresh the image list for this folder if it is currently empty
  CString sCurrentPath{GetPathName()};
  if (theApp.m_ImagesInCurrentFolder.size() == 0)
    theApp.RefreshImageList2(sCurrentPath);

  //Find the current image index if unknown
  if (theApp.m_nCurrentImageIndex == -1)
  {
    const auto iterFind{std::find(theApp.m_ImagesInCurrentFolder.begin(), theApp.m_ImagesInCurrentFolder.end(), sCurrentPath)};
    if (iterFind != theApp.m_ImagesInCurrentFolder.end())
#pragma warning(suppress: 26472 26486)
      theApp.m_nCurrentImageIndex = static_cast<int>(iterFind - theApp.m_ImagesInCurrentFolder.begin());
  }

  //Then display the previous one
  if (theApp.m_nCurrentImageIndex > 0)
  {
    CString sOldPathName{GetPathName()};
    const CDocument* pDoc = theApp.OpenPrevDocumentFile();
    if (pDoc == nullptr)
    {
      //Reset this document to the current image before the failed load above
      CDocTemplate* pTemplate{GetDocTemplate()};
#pragma warning(suppress: 26496)
      AFXASSUME(pTemplate != nullptr);
      if (pTemplate->OpenDocumentFile(sOldPathName, FALSE, TRUE) == nullptr)
        pTemplate->OpenDocumentFile(nullptr, FALSE, TRUE);
    }
  }
  else
  {
    //We've reached the first image, ask if they want to go back to the end
    bool bShowLast{false};
    if (theApp.m_bPromptStartEndFolder)
    {
      GetView()->KillSlideshow();

      CString sContent;
      if (!sContent.LoadString(IDS_REACHED_FIRST_IMAGE_IN_FOLDER))
        return;
      CTaskDialog dlg{sContent, _T(""), theApp.m_pszAppName, TDCBF_YES_BUTTON | TDCBF_NO_BUTTON};
      dlg.SetOptions(TDF_ALLOW_DIALOG_CANCELLATION | TDF_POSITION_RELATIVE_TO_WINDOW);
      CString sDontAskAgain;
      if (!sDontAskAgain.LoadString(IDS_DONT_ASK_AGAIN))
        return;
      dlg.SetVerificationCheckboxText(sDontAskAgain);
      bShowLast = (dlg.DoModal(AfxGetMainWnd()->GetSafeHwnd()) == IDYES);
      theApp.m_bPromptStartEndFolder = !dlg.GetVerificationCheckboxState();
    }
    else
      bShowLast = true;

    if (bShowLast)
      OnViewLastImage();
  }
}

void CEZViewDoc::OnViewRandomImage()
{
  //Display the hourglass as this may take some time
  CWaitCursor wait;

  //Refresh the image list for this folder if it is currently empty
  CString sCurrentPath{GetPathName()};
  if (theApp.m_ImagesInCurrentFolder.size() == 0)
    theApp.RefreshImageList2(sCurrentPath);

  //Work out the next image to show
#pragma warning(suppress: 26472)
  const int nNewIndex{static_cast<int>(rand() % theApp.m_ImagesInCurrentFolder.size())};
  CString sOldPathName{GetPathName()};
  const CDocument* pDoc{theApp.OpenDocumentFile(nNewIndex)};
  if (pDoc == nullptr)
  {
    //Reset this document to the current image before the failed load above
    CDocTemplate* pTemplate{GetDocTemplate()};
#pragma warning(suppress: 26496)
    AFXASSUME(pTemplate != nullptr);
    if (pTemplate->OpenDocumentFile(sOldPathName, FALSE, TRUE) == nullptr)
      pTemplate->OpenDocumentFile(nullptr, FALSE, TRUE);
  }
}

#pragma warning(suppress: 26429)
void CEZViewDoc::OnUpdateViewRandomImage(CCmdUI* pCmdUI)
{
#pragma warning(suppress: 26486 26489)
  pCmdUI->Enable(GetView()->m_pImage != nullptr);
}

#pragma warning(suppress: 26429)
void CEZViewDoc::OnUpdateViewFirstImage(CCmdUI* pCmdUI)
{
#pragma warning(suppress: 26486 26489)
  pCmdUI->Enable(GetView()->m_pImage != nullptr);
}

#pragma warning(suppress: 26429)
void CEZViewDoc::OnUpdateViewLastImage(CCmdUI* pCmdUI)
{
#pragma warning(suppress: 26486 26489)
  pCmdUI->Enable(GetView()->m_pImage != nullptr);
}

#pragma warning(suppress: 26429)
void CEZViewDoc::OnUpdateViewNextImage(CCmdUI* pCmdUI)
{
#pragma warning(suppress: 26486 26489)
  pCmdUI->Enable(GetView()->m_pImage != nullptr);
}

#pragma warning(suppress: 26429)
void CEZViewDoc::OnUpdateViewPreviousImage(CCmdUI* pCmdUI)
{
#pragma warning(suppress: 26486 26489)
  pCmdUI->Enable(GetView()->m_pImage != nullptr);
}

CEZViewView* CEZViewDoc::GetView() const
{
  POSITION posView{GetFirstViewPosition()};
  while (posView != nullptr)
  {
#pragma warning(suppress: 26466)
    auto pView{static_cast<CEZViewView*>(GetNextView(posView))};
    ASSERT(pView != nullptr);
    return pView;
  }
  return nullptr;
}

void CEZViewDoc::OnEditCopy()
{
  //Validate our parameters
#pragma warning(suppress: 26429 26496)
  CEZViewView* pView{GetView()};
  ASSERT(pView != nullptr);
#pragma warning(suppress: 26489)
  ASSERT(pView->m_pImage); //NOLINT(clang-analyzer-core.CallAndMessage)

  //Create a memDC compatible with the view backed by a DIB section
  CWindowDC dc{pView};
  CDC dcMem;
#pragma warning(suppress: 26486)
  if (!dcMem.CreateCompatibleDC(&dc))
    return;
  BITMAPINFOHEADER bih;
  bih.biSize = sizeof(BITMAPINFOHEADER);
#pragma warning(suppress: 26489)
  const CD2DSizeU sizeImage{pView->m_pImage->GetPixelSize()}; //NOLINT(clang-analyzer-core.CallAndMessage)
  bih.biWidth = sizeImage.width;
  bih.biHeight = sizeImage.height;
  bih.biPlanes = 1;
  bih.biBitCount = 32;
  bih.biCompression = BI_RGB;
  bih.biSizeImage = sizeImage.width * sizeImage.height;
  bih.biXPelsPerMeter = 0;
  bih.biYPelsPerMeter = 0;
  bih.biClrUsed = 0;
  bih.biClrImportant = 0;
  void* pBits{nullptr};
#pragma warning(suppress: 26486 26490)
  HBITMAP hDIBSection{CreateDIBSection(dcMem.m_hDC, reinterpret_cast<BITMAPINFO*>(&bih), DIB_RGB_COLORS, &pBits, nullptr, 0)};
  if (hDIBSection == nullptr)
    return;
  CBitmap bmDibSection;
  bmDibSection.Attach(hDIBSection);

  //Select the DIB section into a DC
  dcMem.SelectObject(hDIBSection);

  //Create a DC render target
  CDCRenderTarget dcRenderTarget;
  D2D1_RENDER_TARGET_PROPERTIES props;
  props.type = D2D1_RENDER_TARGET_TYPE_DEFAULT;
  props.pixelFormat.format = DXGI_FORMAT_B8G8R8A8_UNORM;
  props.pixelFormat.alphaMode = D2D1_ALPHA_MODE_IGNORE;
  props.dpiX = props.dpiY = 0;
  props.usage = D2D1_RENDER_TARGET_USAGE_NONE;
  props.minLevel = D2D1_FEATURE_LEVEL_DEFAULT;
  if (!dcRenderTarget.Create(props))
    return;
#pragma warning(suppress: 26486)
  if (!dcRenderTarget.BindDC(dcMem, CRect(0, 0, bih.biWidth, bih.biHeight)))
    return;

  //Draw the image into the DC render target
  dcRenderTarget.BeginDraw();
  const CD2DRectF rectBitmap{0, 0, static_cast<FLOAT>(sizeImage.width), static_cast<FLOAT>(sizeImage.height)};
  if (pView->m_nActivePage != -1)
  {
    ASSERT(pView->m_pDecoder != nullptr);

    //Retrieve the current frame
    ATL::CComPtr<IWICBitmapFrameDecode> pWicFrame;
    HRESULT hr{pView->m_pDecoder->GetFrame(pView->m_nActivePage, &pWicFrame)};
    if (FAILED(hr))
      return;
    auto pD2DState{AfxGetD2DState()};
    if (pD2DState == nullptr)
      return;
    IWICImagingFactory* pFactory{pD2DState->GetWICFactory()};
    if (pFactory == nullptr)
      return;
    ATL::CComPtr<IWICFormatConverter> pConverter;
    hr = pFactory->CreateFormatConverter(&pConverter);
    if (FAILED(hr))
      return;
    //Format convert to 32bppPBGRA which D2D expects
    hr = pConverter->Initialize(pWicFrame, GUID_WICPixelFormat32bppPBGRA, WICBitmapDitherTypeNone, nullptr, 0.0f, WICBitmapPaletteTypeCustom);
    if (FAILED(hr))
      return;

    //Create a D2D bitmap from IWICBitmapSource
#pragma warning(suppress: 26429)
    ID2D1RenderTarget* pRT{dcRenderTarget.GetRenderTarget()};
    ASSERT(pRT != nullptr);
    ID2D1Bitmap* pTempRawFrame{nullptr};
    hr = pRT->CreateBitmapFromWicBitmap(pConverter, nullptr, &pTempRawFrame);
    if (FAILED(hr))
      return;

#pragma warning(suppress: 26409)
    CD2DBitmap frame{&dcRenderTarget, UINT{42}, nullptr, CD2DSizeU(0, 0), FALSE}; //Required because "default" style constructor is protected!
    frame.Attach(pTempRawFrame);

#pragma warning(suppress: 26486)
    dcRenderTarget.DrawBitmap(&frame, rectBitmap);
  }
  else if (pView->m_FrameComposeRT.IsValid())
  {
    //Create the render target
    CBitmapRenderTarget frameComposeRT;
    HRESULT hr{pView->CreateGifFrameComposeRenderTarget(&dcRenderTarget, frameComposeRT)};
    if (FAILED(hr))
      return;

    //Compose one frame
    hr = pView->OverlayNextGifFrameForCopy(&dcRenderTarget, frameComposeRT);
    if (FAILED(hr))
      return;

    CD2DBitmap frameToRender{&dcRenderTarget, UINT{42}, nullptr, CD2DSizeU(0, 0), FALSE}; //Required because "default" style constructor is protected!
    if (!frameComposeRT.GetBitmap(frameToRender))
      return;
    dcRenderTarget.DrawBitmap(&frameToRender, rectBitmap);
  }
  else
  {
    CString sPathName{GetPathName()};
    CString sExt;
    _tsplitpath_s(sPathName, nullptr, 0, nullptr, 0, nullptr, 0, sExt.GetBuffer(_MAX_EXT), _MAX_EXT);
    sExt.ReleaseBuffer();
    if (sExt.CompareNoCase(_T(".QOI")) == 0)
    {
      std::unique_ptr<CD2DBitmap> image;
      pView->LoadQOI(sPathName, dcRenderTarget.GetRenderTarget(), image);
      dcRenderTarget.DrawBitmap(image.get(), rectBitmap);
    }
    else
    {
      CD2DBitmap image{&dcRenderTarget, GetPathName()};
      dcRenderTarget.DrawBitmap(&image, rectBitmap);
    }
  }
  if (FAILED(dcRenderTarget.EndDraw()))
    return;

  //Finally copy the dib section to the clipboard
  if (!pView->OpenClipboard())
    return;
  HGLOBAL hClipboardDib{DDBToDIB(&bmDibSection)};
  if (hClipboardDib == nullptr)
    return;
  EmptyClipboard();
  SetClipboardData(CF_DIB, hClipboardDib);
  CloseClipboard();
}

#pragma warning(suppress: 26429)
void CEZViewDoc::OnUpdateEditCopy(CCmdUI* pCmdUI)
{
#pragma warning(suppress: 26486 26489)
  pCmdUI->Enable(GetView()->m_pImage != nullptr);
}

void CEZViewDoc::OnViewFirstPage()
{
#pragma warning(suppress: 26429)
  CEZViewView* pView{GetView()};
  ASSERT(pView != nullptr);
  pView->m_nActivePage = 0; //NOLINT(clang-analyzer-core.NullDereference)
#pragma warning(suppress: 26489)
  pView->m_pImage.reset();
  pView->InvalidateRect(nullptr, FALSE);
}

#pragma warning(suppress: 26429)
void CEZViewDoc::OnUpdateViewFirstPage(CCmdUI* pCmdUI)
{
  const CEZViewView* pView{GetView()};
  ASSERT(pView != nullptr);
  const bool bEnable{pView->m_ImageType == CEZViewView::IMAGE_TYPE::MULTIPAGE_IMAGE}; //NOLINT(clang-analyzer-core.NullDereference)
  if (bEnable)
  {
#pragma warning(suppress: 26486)
    pCmdUI->Enable(TRUE);
#pragma warning(suppress: 26486)
    pCmdUI->SetCheck(pView->m_nActivePage == 0);
  }
  else
#pragma warning(suppress: 26486)
    pCmdUI->Enable(FALSE);
}

void CEZViewDoc::OnViewLastPage()
{
#pragma warning(suppress: 26429)
  CEZViewView* pView{GetView()};
  ASSERT(pView != nullptr);
  pView->m_nActivePage = pView->m_nFrames - 1; //NOLINT(clang-analyzer-core.NullDereference)
#pragma warning(suppress: 26489)
  pView->m_pImage.reset();
  pView->InvalidateRect(nullptr, FALSE);
}

#pragma warning(suppress: 26429)
void CEZViewDoc::OnUpdateViewLastPage(CCmdUI* pCmdUI)
{
  const CEZViewView* pView{GetView()};
  ASSERT(pView != nullptr);
  const bool bEnable{pView->m_ImageType == CEZViewView::IMAGE_TYPE::MULTIPAGE_IMAGE}; //NOLINT(clang-analyzer-core.NullDereference)
  if (bEnable)
  {
#pragma warning(suppress: 26486)
    pCmdUI->Enable(TRUE);
#pragma warning(suppress: 26472 26486)
    pCmdUI->SetCheck(pView->m_nActivePage == static_cast<int>(pView->m_nFrames - 1));
  }
  else
#pragma warning(suppress: 26486)
    pCmdUI->Enable(FALSE);
}

void CEZViewDoc::OnViewNextPage()
{
#pragma warning(suppress: 26429)
  CEZViewView* pView{GetView()};
  ASSERT(pView != nullptr);
  ++pView->m_nActivePage; //NOLINT(clang-analyzer-core.NullDereference)
#pragma warning(suppress: 26489)
  pView->m_pImage.reset();
  pView->InvalidateRect(nullptr, FALSE);
}

#pragma warning(suppress: 26429)
void CEZViewDoc::OnUpdateViewNextPage(CCmdUI* pCmdUI)
{
  const CEZViewView* pView{GetView()};
  ASSERT(pView != nullptr);
#pragma warning(suppress: 26472 26486)
  pCmdUI->Enable((pView->m_ImageType == CEZViewView::IMAGE_TYPE::MULTIPAGE_IMAGE) && (pView->m_nActivePage < static_cast<int>(pView->m_nFrames - 1))); //NOLINT(clang-analyzer-core.NullDereference)
}

void CEZViewDoc::OnViewPreviousPage()
{
#pragma warning(suppress: 26429)
  CEZViewView* pView{GetView()};
  ASSERT(pView != nullptr);
  --pView->m_nActivePage; //NOLINT(clang-analyzer-core.NullDereference)
#pragma warning(suppress: 26489)
  pView->m_pImage.reset();
  pView->InvalidateRect(nullptr, FALSE);
}

#pragma warning(suppress: 26429)
void CEZViewDoc::OnUpdateViewPreviousPage(CCmdUI* pCmdUI)
{
  const CEZViewView* pView{GetView()};
  ASSERT(pView != nullptr);
#pragma warning(suppress: 26486)
  pCmdUI->Enable((pView->m_ImageType == CEZViewView::IMAGE_TYPE::MULTIPAGE_IMAGE) && (pView->m_nActivePage > 0)); //NOLINT(clang-analyzer-core.NullDereference)
}

void CEZViewDoc::OnViewOpenPage()
{
#pragma warning(suppress: 26429)
  CEZViewView* pView{GetView()};
  ASSERT(pView != nullptr);
  CGotoPageDlg dlg;
  dlg.m_nMaxPageNumber = pView->m_nFrames; //NOLINT(clang-analyzer-core.NullDereference)
  if (dlg.DoModal() != IDOK)
    return;
  pView->m_nActivePage = dlg.m_nPageNumber - 1;
#pragma warning(suppress: 26489)
  pView->m_pImage.reset();
  pView->InvalidateRect(nullptr, FALSE);
}

#pragma warning(suppress: 26429)
void CEZViewDoc::OnUpdateViewOpenPage(CCmdUI* pCmdUI)
{
  const CEZViewView* pView{GetView()};
  ASSERT(pView != nullptr);
#pragma warning(suppress: 26486)
  pCmdUI->Enable(pView->m_ImageType == CEZViewView::IMAGE_TYPE::MULTIPAGE_IMAGE); //NOLINT(clang-analyzer-core.NullDereference)
}

void CEZViewDoc::OnViewStartAnimation()
{
#pragma warning(suppress: 26429)
  CEZViewView* pView{GetView()};
  ASSERT(pView != nullptr);
  if (pView->m_nAnimationTimerID == 0) //NOLINT(clang-analyzer-core.NullDereference)
  {
    if (pView->m_ImageType == CEZViewView::IMAGE_TYPE::MULTIPAGE_IMAGE)
    {
#pragma warning(suppress: 26472)
      if (pView->m_nActivePage == static_cast<int>(pView->m_nFrames - 1))
        pView->m_nActivePage = 0;
      pView->m_nAnimationTimerID = pView->SetTimer(2, theApp.m_nSlideshowInterval * 1000, nullptr);
    }
    else if (pView->m_ImageType == CEZViewView::IMAGE_TYPE::ANIMATED_GIF)
      pView->ComposeNextGifFrame();
  }
  else
    pView->StopAnimation();
}

#pragma warning(suppress: 26429)
void CEZViewDoc::OnUpdateViewStartAnimation(CCmdUI* pCmdUI)
{
  const CEZViewView* pView{GetView()};
  ASSERT(pView != nullptr);
#pragma warning(suppress: 26486)
  if ((pView->m_ImageType == CEZViewView::IMAGE_TYPE::MULTIPAGE_IMAGE) || (pView->m_ImageType == CEZViewView::IMAGE_TYPE::ANIMATED_GIF)) //NOLINT(clang-analyzer-core.NullDereference)
  {
#pragma warning(suppress: 26486)
    pCmdUI->Enable(true);
#pragma warning(suppress: 26486)
    pCmdUI->SetCheck(pView->m_nAnimationTimerID != 0);
  }
  else
#pragma warning(suppress: 26486)
    pCmdUI->Enable(false);
}
